import { createClient } from "redis";

import config from "../config";
import LogLevels from "../enums/log-levels";

export default class RedisClient {
  private _client: any = null;

  constructor () {
    void this.connect().then(() => {
      if (config.LOG_LEVEL === LogLevels.DEBUG) {
        console.debug("Redis connetion stablished");
      }
    });
  }

  private async connect (): Promise<void> {
    this._client = await createClient({ url: config.REDIS_CONN }).connect().catch((err) => {
      console.error("Redis connection error", err);
      throw err;
    });
  }

  private async disconnect (): Promise<void> {
    if (this._client !== null) {
      await this._client.disconnect();
      this._client = null;
    }
  }

  public async store (key: string, value: string, options?): Promise<boolean> {
    let result = false;
    try {
      if (this._client == null) {
        await this.connect();
      }

      await this._client.set(key, value, options);
      result = true;
    } catch (err: any) {
      console.error(err.message);
    } finally {
      await this.disconnect();
    }

    return result;
  }

  public async retrieve (key: string): Promise<string> {
    let result = "";
    try {
      if (this._client == null) {
        await this.connect();
      }

      result = await this._client.get(key);
    } catch (err: any) {
      console.error(err.message);
    } finally {
      await this.disconnect();
    }

    return result;
  }
}
